@@ -30,7 +30,7 @@ use crate::parser::primitives::{
 use crate::parser::string::{quoted_template, unquoted_template};
 use crate::parser::{filename, filename_password, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
-use crate::typing::Count;
+use crate::types::Count;
 
 /// Parse an option in an `[Options]` section.
 pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
@@ -66,11 +66,15 @@ pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
         "location" => option_follow_location(reader)?,
         "location-trusted" => option_follow_location_trusted(reader)?,
         "max-redirs" => option_max_redirect(reader)?,
+        "max-time" => option_max_time(reader)?,
+        "negotiate" => option_negotiate(reader)?,
         "netrc" => option_netrc(reader)?,
         "netrc-file" => option_netrc_file(reader)?,
         "netrc-optional" => option_netrc_optional(reader)?,
+        "ntlm" => option_ntlm(reader)?,
         "output" => option_output(reader)?,
         "path-as-is" => option_path_as_is(reader)?,
+        "pinnedpubkey" => option_pinned_pub_key(reader)?,
         "proxy" => option_proxy(reader)?,
         "repeat" => option_repeat(reader)?,
         "resolve" => option_resolve(reader)?,
@@ -202,6 +206,16 @@ fn option_max_redirect(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::MaxRedirect(value))
 }
 
+fn option_max_time(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = duration_option(reader)?;
+    Ok(OptionKind::MaxTime(value))
+}
+
+fn option_negotiate(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = boolean_option(reader)?;
+    Ok(OptionKind::Negotiate(value))
+}
+
 fn option_netrc(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::NetRc(value))
@@ -217,6 +231,11 @@ fn option_netrc_optional(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::NetRcOptional(value))
 }
 
+fn option_ntlm(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = non_recover(boolean_option, reader)?;
+    Ok(OptionKind::Ntlm(value))
+}
+
 fn option_output(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = filename::parse(reader)?;
     Ok(OptionKind::Output(value))
@@ -227,6 +246,11 @@ fn option_path_as_is(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::PathAsIs(value))
 }
 
+fn option_pinned_pub_key(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = unquoted_template(reader)?;
+    Ok(OptionKind::PinnedPublicKey(value))
+}
+
 fn option_proxy(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = unquoted_template(reader)?;
     Ok(OptionKind::Proxy(value))
@@ -446,7 +470,7 @@ mod tests {
     use super::*;
     use crate::ast::{LineTerminator, Number, Template, TemplateElement, Whitespace, I64};
     use crate::reader::Pos;
-    use crate::typing::ToSource;
+    use crate::types::ToSource;
 
     #[test]
     fn test_option_insecure() {
