@@ -16,7 +16,7 @@
  *
  */
 use crate::ast::{
-    Assert, Capture, Cookie, FileParam, FileValue, MultipartParam, Section, SectionValue,
+    Assert, Capture, Cookie, FilenameParam, FilenameValue, MultipartParam, Section, SectionValue,
     SourceInfo, Whitespace,
 };
 use crate::combinator::{optional, recover, zero_or_more};
@@ -185,7 +185,7 @@ fn cookie(reader: &mut Reader) -> ParseResult<Cookie> {
 fn multipart_param(reader: &mut Reader) -> ParseResult<MultipartParam> {
     let save = reader.cursor();
     match file_param(reader) {
-        Ok(f) => Ok(MultipartParam::FileParam(f)),
+        Ok(f) => Ok(MultipartParam::FilenameParam(f)),
         Err(e) => {
             if e.recoverable {
                 reader.seek(save);
@@ -198,7 +198,7 @@ fn multipart_param(reader: &mut Reader) -> ParseResult<MultipartParam> {
     }
 }
 
-fn file_param(reader: &mut Reader) -> ParseResult<FileParam> {
+fn file_param(reader: &mut Reader) -> ParseResult<FilenameParam> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let key = recover(key_string::parse, reader)?;
@@ -207,7 +207,7 @@ fn file_param(reader: &mut Reader) -> ParseResult<FileParam> {
     let space2 = zero_or_more_spaces(reader)?;
     let value = file_value(reader)?;
     let line_terminator0 = line_terminator(reader)?;
-    Ok(FileParam {
+    Ok(FilenameParam {
         line_terminators,
         space0,
         key,
@@ -218,7 +218,7 @@ fn file_param(reader: &mut Reader) -> ParseResult<FileParam> {
     })
 }
 
-fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
+fn file_value(reader: &mut Reader) -> ParseResult<FilenameValue> {
     try_literal("file,", reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let f = filename::parse(reader)?;
@@ -252,7 +252,7 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
         }
     };
 
-    Ok(FileValue {
+    Ok(FilenameValue {
         space0,
         filename: f,
         space1,
@@ -270,8 +270,17 @@ fn capture(reader: &mut Reader) -> ParseResult<Capture> {
     let space2 = zero_or_more_spaces(reader)?;
     let q = query(reader)?;
     let filters = filters(reader)?;
-    let space3 = zero_or_more_spaces(reader)?;
-    let redact = try_literal("redact", reader).is_ok();
+    let (redacted, space3) = if let Some(ws) = optional(redacted, reader)? {
+        (true, ws)
+    } else {
+        // No `redact` keywork, space3 is an empty whitespace and the optional remaining whitespace
+        // will be consumed by the line terminator.
+        let pos = reader.cursor().pos;
+        let value = String::new();
+        let source_info = SourceInfo::new(pos, pos);
+        (false, Whitespace { value, source_info })
+    };
+
     let line_terminator0 = line_terminator(reader)?;
     Ok(Capture {
         line_terminators,
@@ -282,11 +291,17 @@ fn capture(reader: &mut Reader) -> ParseResult<Capture> {
         query: q,
         filters,
         space3,
-        redact,
+        redacted,
         line_terminator0,
     })
 }
 
+fn redacted(reader: &mut Reader) -> ParseResult<Whitespace> {
+    let space = zero_or_more_spaces(reader)?;
+    try_literal("redact", reader)?;
+    Ok(space)
+}
+
 fn assert(reader: &mut Reader) -> ParseResult<Assert> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
@@ -314,7 +329,8 @@ mod tests {
         KeyValue, LineTerminator, Number, Predicate, PredicateFunc, PredicateFuncValue,
         PredicateValue, Query, QueryValue, Template, TemplateElement, I64,
     };
-    use crate::typing::ToSource;
+    use crate::reader::CharPos;
+    use crate::types::ToSource;
 
     #[test]
     fn test_section_name() {
@@ -485,7 +501,7 @@ mod tests {
         let mut reader = Reader::new("file,hello.txt;");
         assert_eq!(
             file_value(&mut reader).unwrap(),
-            FileValue {
+            FilenameValue {
                 space0: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
@@ -512,7 +528,7 @@ mod tests {
         let mut reader = Reader::new("file,hello.txt; text/html");
         assert_eq!(
             file_value(&mut reader).unwrap(),
-            FileValue {
+            FilenameValue {
                 space0: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
@@ -551,7 +567,7 @@ mod tests {
         let file_value = file_value(&mut reader).unwrap();
         let content_type = file_value.content_type.unwrap();
         assert_eq!(content_type.to_string(), "text/html".to_string());
-        assert_eq!(reader.cursor().index, 25);
+        assert_eq!(reader.cursor().index, CharPos(25));
 
         let mut reader = Reader::new("file,------; text/plain; charset=us-ascii");
         let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
@@ -560,13 +576,13 @@ mod tests {
             content_type.to_string(),
             "text/plain; charset=us-ascii".to_string()
         );
-        assert_eq!(reader.cursor().index, 41);
+        assert_eq!(reader.cursor().index, CharPos(41));
 
         let mut reader = Reader::new("file,******; text/html # comment");
         let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
         let content_type = file_value.content_type.unwrap();
         assert_eq!(content_type.to_string(), "text/html".to_string());
-        assert_eq!(reader.cursor().index, 22);
+        assert_eq!(reader.cursor().index, CharPos(22));
 
         let mut reader = Reader::new("file,{{some_file}}; application/vnd.openxmlformats-officedocument.wordprocessingml.document # comment");
         let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
@@ -575,7 +591,7 @@ mod tests {
             content_type.to_string(),
             "application/vnd.openxmlformats-officedocument.wordprocessingml.document".to_string()
         );
-        assert_eq!(reader.cursor().index, 91);
+        assert_eq!(reader.cursor().index, CharPos(91));
 
         let mut reader = Reader::new("file,{{some_file}}; {{some_content_type}} # comment");
         let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
@@ -584,7 +600,7 @@ mod tests {
             content_type.to_string(),
             "{{some_content_type}}".to_string()
         );
-        assert_eq!(reader.cursor().index, 41);
+        assert_eq!(reader.cursor().index, CharPos(41));
     }
 
     #[test]
@@ -593,40 +609,146 @@ mod tests {
         let capture0 = capture(&mut reader).unwrap();
 
         assert_eq!(
-            capture0.name,
-            Template::new(
-                None,
-                vec![TemplateElement::String {
-                    value: "url".to_string(),
-                    source: "url".to_source(),
-                }],
-                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4))
-            ),
-        );
-        assert_eq!(
-            capture0.query,
-            Query {
-                source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 23)),
-                value: QueryValue::Header {
-                    space0: Whitespace {
-                        value: String::from(" "),
-                        source_info: SourceInfo::new(Pos::new(1, 12), Pos::new(1, 13)),
+            capture0,
+            Capture {
+                line_terminators: vec![],
+                space0: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 1),
+                        end: Pos::new(1, 1),
                     },
-                    name: Template::new(
-                        Some('"'),
-                        vec![TemplateElement::String {
-                            value: "Location".to_string(),
-                            source: "Location".to_source(),
-                        }],
-                        SourceInfo::new(Pos::new(1, 13), Pos::new(1, 23))
-                    )
-                }
+                },
+                name: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "url".to_string(),
+                        source: "url".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4))
+                ),
+                space1: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 4),
+                        end: Pos::new(1, 4),
+                    }
+                },
+                space2: Whitespace {
+                    value: " ".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 5),
+                        end: Pos::new(1, 6),
+                    }
+                },
+                query: Query {
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 23)),
+                    value: QueryValue::Header {
+                        space0: Whitespace {
+                            value: String::from(" "),
+                            source_info: SourceInfo::new(Pos::new(1, 12), Pos::new(1, 13)),
+                        },
+                        name: Template::new(
+                            Some('"'),
+                            vec![TemplateElement::String {
+                                value: "Location".to_string(),
+                                source: "Location".to_source(),
+                            }],
+                            SourceInfo::new(Pos::new(1, 13), Pos::new(1, 23))
+                        )
+                    }
+                },
+                filters: vec![],
+                space3: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 23), Pos::new(1, 23)),
+                },
+                redacted: false,
+                line_terminator0: LineTerminator {
+                    space0: Whitespace {
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 23), Pos::new(1, 23)),
+                    },
+                    comment: None,
+                    newline: Whitespace {
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 23), Pos::new(1, 23)),
+                    },
+                },
             }
         );
 
-        let mut reader = Reader::new("url: header \"Token\" redact");
+        let mut reader = Reader::new("url: header \"Token\"    redact");
         let capture0 = capture(&mut reader).unwrap();
-        assert!(capture0.redact);
+        assert_eq!(
+            capture0,
+            Capture {
+                line_terminators: vec![],
+                space0: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 1),
+                        end: Pos::new(1, 1),
+                    },
+                },
+                name: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "url".to_string(),
+                        source: "url".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4))
+                ),
+                space1: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 4),
+                        end: Pos::new(1, 4),
+                    }
+                },
+                space2: Whitespace {
+                    value: " ".to_string(),
+                    source_info: SourceInfo {
+                        start: Pos::new(1, 5),
+                        end: Pos::new(1, 6),
+                    }
+                },
+                query: Query {
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 20)),
+                    value: QueryValue::Header {
+                        space0: Whitespace {
+                            value: String::from(" "),
+                            source_info: SourceInfo::new(Pos::new(1, 12), Pos::new(1, 13)),
+                        },
+                        name: Template::new(
+                            Some('"'),
+                            vec![TemplateElement::String {
+                                value: "Token".to_string(),
+                                source: "Token".to_source(),
+                            }],
+                            SourceInfo::new(Pos::new(1, 13), Pos::new(1, 20))
+                        )
+                    }
+                },
+                filters: vec![],
+                space3: Whitespace {
+                    value: "    ".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 20), Pos::new(1, 24)),
+                },
+                redacted: true,
+                line_terminator0: LineTerminator {
+                    space0: Whitespace {
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 30), Pos::new(1, 30)),
+                    },
+                    comment: None,
+                    newline: Whitespace {
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 30), Pos::new(1, 30)),
+                    },
+                },
+            }
+        );
     }
 
     #[test]
@@ -654,7 +776,7 @@ mod tests {
                 }
             }
         );
-        assert_eq!(reader.cursor().index, 43);
+        assert_eq!(reader.cursor().index, CharPos(43));
     }
 
     #[test]
@@ -699,8 +821,8 @@ mod tests {
         let mut reader = Reader::new("name: jsonpath \"$.name\"          # name");
         let capture0 = capture(&mut reader).unwrap();
         assert!(capture0.filters.is_empty());
-        assert_eq!(capture0.space3.as_str(), "          ");
-        assert_eq!(capture0.line_terminator0.space0.as_str(), "");
+        assert!(capture0.space3.value.is_empty());
+        assert_eq!(capture0.line_terminator0.space0.as_str(), "          ");
     }
 
     #[test]
